import java.util.Iterator;

public class MyArrayList<T> implements IList<T> {
    private int size;
    private T[] array;
    
    public MyArrayList() {
        this.size = 0;
        this.array = this.makeArray(10);
    }
    
    /**
     * See 'makeArrayOfPairs' method in your ArrayDictionary for more details
     */
    @SuppressWarnings("unchecked")
    private T[] makeArray(int size) {
        return (T[]) new Object[size];
    }
    
    @Override
    public void add(T item) {
        this.insert(size, item);
    }

    @Override
    public T remove() {
        if (this.size == 0) {
            throw new EmptyContainerException();
        }
        return this.delete(this.size - 1);
    }

    @Override
    public T get(int index) {
        this.ensureInBounds(index, this.size - 1);
        return this.array[index];
    }

    @Override
    public void set(int index, T item) {
        this.ensureInBounds(index, this.size - 1);
        this.array[index] = item;
    }

    @Override
    public void insert(int index, T item) {
        this.ensureInBounds(index,  this.size);
        
        if (this.size == this.array.length) {
            T[] newArray = this.makeArray(this.size * 2);
            this.copyFrom(this.array, newArray);
            for (int i = 0; i < this.size; i++) {
                newArray[i] = this.array[i];
            }
            this.array = newArray;
        }
        for (int i = this.size - 1; i >= index; i--) {
            this.array[i + 1] = this.array[i];
        }
        this.array[index] = item;
        this.size += 1;
    }

    @Override
    public T delete(int index) {
        T out = this.array[index];
        for (int i = index; i < this.size - 1; i++) {
            this.array[i] = this.array[i + 1];
        }
        this.size -= 1;
        return out;
    }

    @Override
    public int indexOf(T item) {
        for (int i = 0; i < this.array.length; i++) {
            if (this.array[i].equals(item)) {
                return i;
            }
        }
        return -1;
    }

    @Override
    public int size() {
        return this.size;
    }

    @Override
    public boolean contains(T other) {
        return this.indexOf(other) != -1;
    }
    
    private void ensureInBounds(int index, int max) {
        if (index < 0 || index > max) {
            throw new IndexOutOfBoundsException();
        }
    }

    private void copyFrom(T[] src, T[] dst) {
        for (int i = 0; i < src.length; i++) {
            dst[i] = src[i];
        }
    }

    @Override
    public Iterator<T> iterator() {
        Iterator<T> iter = new MyArrayListIterator<>(this.array, this.size);
        return iter;
    }
}
